//#include "stdafx.h"
#include <dinput.h>
#include "../../input.h"

static LPDIRECTINPUT       g_pDI       = NULL;
static LPDIRECTINPUTDEVICE g_pKeyboard = NULL;
static 	BYTE    diks[256];   // DirectInput keyboard state buffer

const int DIKeyTable[] = {
	 0  ,  033, '1', '2', '3', '4', '5', '6', '7', '8',
	 '9',  '0', '-', '=', 010, '\t','q', 'w', 'e', 'r',
	 't',  'y', 'u', 'i', 'o', 'p', '[', ']', '\r',0x102,
	 'a',  's', 'd', 'f', 'g', 'h', 'j', 'k', 'l', ';',
     0  ,   0 , 0x101,010,'z', 'x', 'c', 'v', 'b', 'n',
	 'm',  ',', '.','/',0x101,  0,   0, ' ',  0 , 0x131,
	 0x132,0x133,0x134,0x135,0x136,0x137,0x138,0x139,0x13a,  0,
     0x1f3,0x147,0x148,0x149,0x14b,0x144,0x145,0x146,0x14a,0x141,
	 0x142,0x143,0x14e,    0,0x13b,0x13c,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,

	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	     0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
	 0x10a,0x124,    0,0x10d,    0,0x10c,    0,0x123,0x10b,    0,

	 0x121,  127,
};


void ev_poll()
{
	int i;
	static BYTE diks_prev[256];
	event_t ev;

	UpdateDirectInput();

	for (i = 0; i < 256; i++)
	{
        if( (diks[i] & 0x80) && !diks_prev[i]){
			ev.type = EV_PRESS;
			ev.code = DIKeyTable[i];
			ev_postevent(&ev);
		}else if( !(diks[i] & 0x80) && diks_prev[i]){
			ev.type = EV_RELEASE;
			ev.code = DIKeyTable[i];
			ev_postevent(&ev);
		}
		diks_prev[i] = diks[i] & 0x80;
	}
}

HRESULT SetAcquire(void)
{
    if( NULL == g_pKeyboard )
        return S_FALSE;
    g_pKeyboard->lpVtbl->Acquire(g_pKeyboard);

    return S_OK;
}

HRESULT InitDirectInput( HWND hWnd )
{
    HRESULT hr;

    hr = DirectInputCreate( (HINSTANCE)GetWindowLong( hWnd, GWL_HINSTANCE ),
		                    DIRECTINPUT_VERSION, &g_pDI, NULL );
    if( FAILED(hr) )
        return hr;

    // Obtain an interface to the system keyboard device.
    hr = g_pDI->lpVtbl->CreateDevice(g_pDI, &GUID_SysKeyboard, &g_pKeyboard, NULL );
    if( FAILED(hr) )
        return hr;
    hr = g_pKeyboard->lpVtbl->SetDataFormat(g_pKeyboard, &c_dfDIKeyboard );
    if( FAILED(hr) )
        return hr;
    hr = g_pKeyboard->lpVtbl->SetCooperativeLevel(g_pKeyboard, hWnd,
                       DISCL_NONEXCLUSIVE | DISCL_BACKGROUND);
//                                     DISCL_NONEXCLUSIVE | DISCL_FOREGROUND );
    if( FAILED(hr) )
        return hr;

	SetAcquire();
    return S_OK;
}


HRESULT UpdateInputState( HWND hDlg )
{
    CHAR strNewText[128];   // Current keyboard state text
    CHAR* str = strNewText;
	int i;

    if( g_pKeyboard )
    {
        HRESULT hr;

        hr = DIERR_INPUTLOST;

        // If input is lost then acquire and keep trying
        while( DIERR_INPUTLOST == hr )
        {
            // Get the input's device state, and put the state in dims
            hr = g_pKeyboard->lpVtbl->GetDeviceState(g_pKeyboard, sizeof(diks), &diks );

            if( hr == DIERR_INPUTLOST )
            {
                hr = g_pKeyboard->lpVtbl->Acquire(g_pKeyboard);
            }
        }

        if( FAILED(hr) )
            return hr;

        // Display the index values of the keys that are down
        for( i = 0; i < 256; i++ )
        {
            if( diks[i] & 0x80 )
                str += wsprintf( str, "%02x ", i );
        }
        *str = 0;   // Terminate the string

    }

    return S_OK;
}

int UpdateDirectInput(void)
{
	HRESULT hr;

    if( g_pKeyboard )
    {

        hr = DIERR_INPUTLOST;
        // If input is lost then acquire and keep trying
        while( DIERR_INPUTLOST == hr )
        {
            // Get the input's device state, and put the state in dims
            hr = g_pKeyboard->lpVtbl->GetDeviceState(g_pKeyboard, sizeof(diks), &diks );

            if( hr == DIERR_INPUTLOST )
                hr = g_pKeyboard->lpVtbl->Acquire(g_pKeyboard);
        }

        if( FAILED(hr) )
            return 0;
    }

	return 0;
}


HRESULT FreeDirectInput()
{
    // Unacquire and release any DirectInputDevice objects.
    if( g_pKeyboard )
    {
        g_pKeyboard->lpVtbl->Unacquire(g_pKeyboard);
        g_pKeyboard->lpVtbl->Release(g_pKeyboard);
        g_pKeyboard = NULL;
    }

    // Release any DirectInput objects.
    if( g_pDI )
    {
        g_pDI->lpVtbl->Release(g_pDI);
        g_pDI = NULL;
    }

    return S_OK;
}

