#include <windows.h>
#include <windowsx.h>
#include "hspdll.h"


extern "C" {
int	 gnuboy_main(int argc, char **argv);
void sys_sanitize(char *s);
void loader_init(char *s);
void loader_unload();
void emu_reset();
void emu_run_frame(int );
void vid_init();
void vid_set_ptr(BYTE *);
void vid_set_pelsize(int);
void vid_close();
BYTE *vid_get_gbscreen();
//void vid_set_pos(int, int, int, int, int);
void pcm_close();
void pcm_init();
void pcm_reinit();
void pcm_start();
void pcm_pause();
char *rc_getstr(char *name);
void state_save(int n);
void state_load(int n);

HRESULT FreeDirectInput();
HRESULT InitDirectInput( HWND hWnd );
}

struct hspboy
{
	int 	Bpp;
	int		Skip;
	DWORD	TimerRes;
	BOOL	Emulation;
	BOOL	RomLoaded;
	HWND	hwnd;
	RECT	Rect;
	BYTE	*gb_screen;
	HDC		hBmpDC;
	HBITMAP	hBmp;
	BITMAPINFO Bi;
};

static struct hspboy Hspboy;
HWND hWndMain;

int WINAPI DllMain (HINSTANCE hInstance, DWORD fdwReason, PVOID pvReserved)
{
     return TRUE ;
}

EXPORT BOOL WINAPI gb_init(BMSCR *p1, int p2, int p3, int p4 )
{
	char module[256];
	char *argv[2];
	TIMECAPS tc;
	static BOOL first = TRUE;

	if(!first)
		return -1;
	first = FALSE;

	Hspboy.Rect.top = Hspboy.Rect.left = 0;
	Hspboy.Rect.bottom = 144;
	Hspboy.Rect.right  = 160;
	Hspboy.Bpp = p1->palmode ? 1 : 3;
	Hspboy.hwnd = p1->hwnd;
	Hspboy.Skip = 0;
	Hspboy.Bi.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
	Hspboy.Bi.bmiHeader.biWidth = 160;
	Hspboy.Bi.bmiHeader.biHeight = 144;
	Hspboy.Bi.bmiHeader.biPlanes = 1;
	Hspboy.Bi.bmiHeader.biBitCount = Hspboy.Bpp * 8;
	Hspboy.Bi.bmiHeader.biCompression = BI_RGB;
	// Ô߂ɐmBITMAPINFO擾Ă
	GetDIBits(Hspboy.hBmpDC,(HBITMAP)Hspboy.hBmpDC,0,400,0,&Hspboy.Bi,DIB_RGB_COLORS);
	Hspboy.hBmp = CreateDIBSection( NULL, &Hspboy.Bi , DIB_RGB_COLORS, (void**)&Hspboy.gb_screen, NULL, 0 );
	Hspboy.hBmpDC = CreateCompatibleDC(NULL);
	SelectObject(Hspboy.hBmpDC, Hspboy.hBmp);
	Hspboy.Emulation = FALSE;
	Hspboy.RomLoaded = FALSE;
	hWndMain = Hspboy.hwnd;

	if( timeGetDevCaps(&tc, sizeof(TIMECAPS))== TIMERR_NOERROR)
	{
		Hspboy.TimerRes = tc.wPeriodMin;
		timeBeginPeriod( Hspboy.TimerRes);
	}
	vid_set_ptr(Hspboy.gb_screen);
	vid_set_pelsize(Hspboy.Bpp);
	vid_init();
	pcm_init();
	InitDirectInput(Hspboy.hwnd);
	GetModuleFileName(NULL, module, sizeof(char)*256);
//	MessageBox(NULL, module, module, MB_OK);
	argv[0] = module;
	gnuboy_main(1, argv);
	CreateDirectory(rc_getstr("savedir"), NULL);

	return 0;
}

EXPORT BOOL WINAPI gb_start(int p1, char *p2, int p3, int p4 )
{
	if(Hspboy.RomLoaded == TRUE)
		Hspboy.Emulation = TRUE;
	if(Hspboy.Emulation == TRUE)
		pcm_start();
	return 0;
}

EXPORT BOOL WINAPI gb_stop(int p1, char *p2, int p3, int p4 )
{
	if(Hspboy.Emulation == TRUE)
		pcm_pause();
	Hspboy.Emulation = FALSE;

	return 0;
}

EXPORT BOOL WINAPI gb_loadrom(BMSCR *p1, char *p2, int p3, int p4 )
{
	char *rom;

	if(Hspboy.RomLoaded == TRUE)
		loader_unload();

	rom = strdup(p2);
	sys_sanitize(rom);

	loader_init(rom);
	emu_reset();

	Hspboy.RomLoaded = TRUE;
	return 0;
}

EXPORT BOOL WINAPI gb_run_frame(BMSCR *p1, int p2, int p3, int p4 )
{
	HDC hdc;
	HPALETTE opal;
	static int fskip;

	if(Hspboy.Emulation != TRUE)
		return 1;

	if(fskip >= Hspboy.Skip){
		emu_run_frame(1);
		fskip = 0;
	}else{
		emu_run_frame(0);
		fskip++;
	}


	if (p1->fl_udraw==0) return 0;
	hdc=GetDC( p1->hwnd );
	if (p1->hpal!=NULL) {
		opal=SelectPalette( hdc, p1->hpal, 0 );
		RealizePalette( hdc );
	}
	StretchBlt( hdc, Hspboy.Rect.left, Hspboy.Rect.top,
				Hspboy.Rect.right, Hspboy.Rect.bottom, Hspboy.hBmpDC, 0, 0, 160, 144, SRCCOPY );
	if (p1->hpal!=NULL) {
		SelectPalette( hdc, opal, 0 );
	}
	ReleaseDC( p1->hwnd,hdc );
	return 0;
}

EXPORT BOOL WINAPI gb_vid_setpos(int p1, int p2, int p3, int p4 )
{
	Hspboy.Rect.left   = p1;
	Hspboy.Rect.top    = p2;
	Hspboy.Rect.right  = p3;
	Hspboy.Rect.bottom = p4;

	return 0;
}

EXPORT BOOL WINAPI gb_setskip(int p1, int p2, int p3, int p4 )
{
	Hspboy.Skip = p1;
	return 0;
}

EXPORT BOOL WINAPI gb_reset(int p1, int p2, int p3, int p4 )
{
	emu_reset();
	return 0;
}

EXPORT BOOL WINAPI gb_savestate(int p1, int p2, int p3, int p4 )
{
	state_save(p1);
	return 0;
}

EXPORT BOOL WINAPI gb_loadstate(int p1, int p2, int p3, int p4 )
{
	state_load(p1);
	return 0;
}

EXPORT BOOL WINAPI gb_snd_close(int p1, int p2, int p3, int p4 )
{
	pcm_close();
	return 0;
}

EXPORT BOOL WINAPI gb_snd_open(int p1, int p2, int p3, int p4 )
{
	pcm_reinit();
	return 0;
}

EXPORT BOOL WINAPI gb_clean(int p1, int p2, int p3, int p4 )
{
	if(Hspboy.TimerRes){
		timeEndPeriod(Hspboy.TimerRes);
		Hspboy.TimerRes = 0;
	}

	if(Hspboy.hBmpDC){
		DeleteObject(Hspboy.hBmpDC);
		Hspboy.hBmpDC = NULL;
	}
	if(Hspboy.hBmp){
		DeleteBitmap(Hspboy.hBmp);
		Hspboy.hBmp = NULL;
	}

	loader_unload();
	vid_close();
	pcm_close();
	FreeDirectInput();

	return 0;
}

